#!/usr/bin/python
#
# This software is in the public domain, furnished "as is", without technical
# support, and with no warranty, express or implied, as to its usefulness for
# any purpose.
#
# delugecontrol.py
# Allows changing diverse configuration parameter for the Deluge daemon running on localhost
# Take a look at the code to see what parameters can be changed (c'mon, it's less than 130 lines,
# and most of them are comments!)
#
# Based on a script for previous Deluge versions by: Garett Harnish
#
# Author (more like a remixer): Ivan Fernandez


from sys import argv, exit, stderr
from optparse import OptionParser

# Import the client module
from deluge.ui.client import client
# Import the reactor module from Twisted - this is for our mainloop
from twisted.internet import reactor

# Set up the logger to print out errors
from deluge.log import setupLogger

import logging

setupLogger()

def isFloatDigit (string):
    if string.isdigit():
        return True
    else:
        try:
            tmp = float(string)
            return True
        except: return False;

# set up command-line options
parser = OptionParser()
parser.add_option("--max_active_limit", help="sets the absolute maximum number of active torrents on the deluge backend", dest="max_active_limit")
parser.add_option("--max_active_downloading", help="sets the maximum number of active downloading torrents on the deluge backend", dest="max_active_downloading")
parser.add_option("--max_active_seeding", help="sets the maximum number of active seeding torrents on the deluge backend", dest="max_active_seeding")
parser.add_option("--max_download_speed", help="sets the maximum global download speed on the deluge backend", dest="max_download_speed")
parser.add_option("--max_upload_speed", help="sets the maximum global upload speed on the deluge backend", dest="max_upload_speed")
parser.add_option("--debug", help="outputs debug information to the console", default=False, action="store_true", dest="debug")

# grab command-line options
(options, args) = parser.parse_args()

if not options.debug:
    logging.disable(logging.ERROR)

# Connect to a daemon running on the localhost
# We get a Deferred object from this method and we use this to know if and when
# the connection succeeded or failed.
d = client.connect()
settings = {}

# set values if set and valid
if options.max_active_limit:
    if options.max_active_limit.isdigit() and int(options.max_active_limit) >= 0:
        settings['max_active_limit'] = int(options.max_active_limit)
    else:
        stderr.write ("ERROR: Invalid max_active_limit parameter!\n")
        exit (-1)

if options.max_active_downloading:
    if options.max_active_downloading.isdigit() and int(options.max_active_downloading) >= 0:
        settings['max_active_downloading'] = int(options.max_active_downloading)
    else:
        stderr.write ("ERROR: Invalid max_active_downloading parameter!\n")
        exit (-1)

if options.max_active_seeding:
    if options.max_active_seeding.isdigit() and int(options.max_active_seeding) >= 0:
        settings['max_active_seeding'] = int(options.max_active_seeding)
    else:
        stderr.write ("ERROR: Invalid max_active_seeding parameter!\n")
        exit (-1)

if options.max_download_speed:
    if isFloatDigit(options.max_download_speed) and (float(options.max_download_speed) >= 0.0 or float(options.max_download_speed) == -1.0):
        settings['max_download_speed'] = float(options.max_download_speed)
    else:
        stderr.write ("ERROR: Invalid max_download_speed parameter!\n")
        exit (-1)

if options.max_upload_speed:
    if isFloatDigit(options.max_upload_speed) and (float(options.max_upload_speed) >= 0.0 or float(options.max_upload_speed) == -1.0):
        settings['max_upload_speed'] = float(options.max_upload_speed)
    else:
        stderr.write ("ERROR: Invalid max_upload_speed parameter!\n")
        exit (-1)

# We create a callback function to be called upon a successful connection
def on_connect_success(result):
    print "Connection was successful!"
    def on_set_config(value,key):
        print "Daemon config parameter modified!"
        print "%s" % (key)
        # Disconnect from the daemon once we've got what we needed
        client.disconnect()
        # Stop the twisted main loop and exit
        reactor.stop()
    
    # Request the config value for the key 'download_location'   
    if settings:
       client.core.set_config(settings).addCallback(on_set_config, settings)
    else:
      print "But no valid parameters were suppplied..."  
      reactor.stop()


# We add the callback to the Deferred object we got from connect()
d.addCallback(on_connect_success)

# We create another callback function to be called when an error is encountered
def on_connect_fail(result):
    print "Connection failed!"
    print "result:", result

# We add the callback (in this case it's an errback, for error)
d.addErrback(on_connect_fail)

# Run the twisted main loop to make everything go
reactor.run()

